set define '^'
set verify off
prompt ...wwv_flow_team_api


Rem  Copyright (c) Oracle Corporation 2010. All Rights Reserved.
Rem
Rem    NAME
Rem      wwv_flow_team_api.sql
Rem
Rem    DESCRIPTION
Rem      Team development API package
Rem
Rem    MODIFIED (MM/DD/YYYY)
Rem     mhichwa  01/20/2010 - Created
Rem     mhichwa  01/25/2010 - extended
Rem     mhichwa  01/26/2010 - implemented delete apis
Rem     pawolf   04/08/2010 - Added APIs for import
Rem     jkallman 05/14/2010 - Updated c_compatable_from_version to 2010.05.13
Rem     jstraub  11/09/2012 - Added create_team_tags
Rem     asitters 05/20/2014 - Added overloaded create_todo procedure that includes p_id parameter. Used by 4000:261 (see Feature #1415)
Rem     msewtz   08/17/2015 - Updated APIs to support DDL changes for approval status, dependent features and links (feature 1825)
Rem     msewtz   01/10/2017 - Added p_feature_type_id to second version of create_feature (bug 25374827)


create or replace package wwv_flow_team_api
as

c_compatable_from_version constant varchar2(255) := '2010.05.13';

-- used by the import
g_id_offset number := 0;

procedure check_version (
    --
    -- This call is expected to be made before running any procedure within wwv_flow_team_api.
    -- This "check_version" procedure identifies the version of team development that created your export.
    --
    -- Example:
    --
    --    begin
    --       wwv_flow_team_api.set_version(p_version_yyyy_mm_dd=>'2001.08.27');
    --    end;
    --    /
    --
    -- This call is generated by the wwv_flow_team_gen_api.export procedure.
    -- All team development versions are in the form YYYY.MM.DD.
    -- No API calls will work if the versions are incompatable.
    -- An incompatable version is defined as the wwv_flow_team_api.c_compatable_from_version
    -- (a plsql package constant that indicates from which date this api is good from)
    -- less then or equal to the calling version passed to this procedure.
    --
    p_version_yyyy_mm_dd in varchar2 );

procedure create_bug (
    -- bug
    p_title                     in varchar2  default null,
    p_severity                  in number    default null,
    p_status                    in number    default null,
    p_priority                  in number    default null,
    --
    -- resolution
    p_assigned_to               in varchar2  default null,
    p_new_assigned_to           in varchar2  default null,
    p_fix_by_release            in varchar2  default null,
    p_new_release               in varchar2  default null,
    p_actual_fix_date           in date      default null,
    p_target_milestone_id       in number    default null,
    --
    -- description
    p_description               in clob      default null,
    p_application_id            in number    default null,
    p_page_id                   in number    default null,
    --
    -- other
    p_component                 in varchar2  default null,
    p_tags                      in varchar2  default null,
    p_estimated_fix_date        in date      default null,
    p_reported_platform         in varchar2  default null,
    p_reported_browser          in varchar2  default null,
    p_reported_operating_system in varchar2  default null,
    p_duplicate_of_bug          in number    default null,
    p_customer_name             in varchar2  default null,
    p_customer_issue            in clob      default null,
    p_product                   in varchar2  default null,
    p_product_version           in varchar2  default null,
    p_impact                    in varchar2  default null,
    --
    -- related
    p_feature_id                in number    default null,
    p_task_id                   in number    default null
    );

procedure delete_bug (
    p_id in number );

procedure create_todo (
    -- todo
    p_to_do_action              in varchar2  default null,
    p_assigned_to               in varchar2  default null,
    p_new_assigned_to           in varchar2  default null,
    p_contributor               in varchar2  default null,
    p_new_contributor           in varchar2  default null,
    p_parent_to_do_id           in number    default null,
    p_status                    in number    default null,
    --
    -- dates
    p_due_date                  in date      default null,
    p_start_date                in date      default null,
    p_date_completed            in date      default null,
    --
    -- details
    p_category                  in varchar2  default null,
    p_new_category              in varchar2  default null,
    p_release                   in varchar2  default null,
    p_new_release               in varchar2  default null,
    p_description               in clob      default null,
    p_feature_id                in number    default null,
    p_milestone_id              in number    default null,
    --
    -- customer details
    p_customer_details          in varchar2  default null,
    --
    -- tags
    p_tags                      in varchar2  default null,
    --
    -- application context
    p_application_id            in number    default null,
    p_page_id                   in number    default null,
    p_product_version           in varchar2  default null,
    --
    -- additional details
    p_ref_component_type        in varchar2  default null,
    p_group                     in varchar2  default null,
    p_estimated_effort_in_hours in number    default null
    );

procedure delete_todo (
    p_id in number default null);

procedure create_feature_type (
    p_id                                number,
    p_type_id                           number default null,
    p_type_name                         varchar2 default null, 
    p_type_desc                         varchar2 default null, 
    p_reporting_default_yn              varchar2 default null); 

procedure create_feature (
    --
    -- feature
    --
    p_feature_name              in varchar2  default null,
    p_feature_type_id           in number    default null,
    p_feature_tags              in varchar2  default null,
    --
    p_feature_owner             in varchar2  default null,
    p_new_feature_owner         in varchar2  default null,
    --
    p_contributor               in varchar2  default null,
    p_new_contributor           in varchar2  default null,
    --
    p_focus_area                in varchar2  default null,
    p_new_focus_area            in varchar2  default null,
    --
    p_release                   in varchar2  default null,
    p_new_release               in varchar2  default null,
    --
    p_status                    in number    default null,
    p_desirability              in number    default null,
    p_priority                  in number    default null,
    p_milestone_id              in number    default null,
    p_parent_feature_id         in number    default null,
    --
    -- dates
    --
    p_start_date                in date      default null,
    p_due_date                  in date      default null,
    --
    -- feature summary
    --
    p_publishable_description   in varchar2  default null,
    p_publishable_yn            in varchar2  default null,
    --
    -- feature details
    --
    p_description               in clob      default null,
    p_justification             in varchar2  default null,
    p_application_id            in number    default null,
    p_module                    in varchar2  default null,
    p_estimated_effort_in_hours in number    default null,
    --
    -- documentation
    --
    p_doc_impact                in varchar2  default null,
    p_doc_status                in number    default null,
    p_doc_writer                in varchar2  default null,
    --
    -- testing
    --
    p_testing_impact            in varchar2  default null,
    p_testing_status            in number    default null,
    p_testing_assignee          in varchar2  default null,
    --
    -- security
    --
    p_security_impact           in varchar2  default null,
    p_security_status           in number    default null,
    p_security_assignee         in varchar2  default null,
    --
    -- accessibility
    --
    p_accessibility_impact      in varchar2  default null,
    p_accessibility_status      in number    default null,
    p_accessibility_assignee    in varchar2  default null,
    --
    -- globalization
    --
    p_globalization_impact      in varchar2  default null,
    p_globalization_status      in number    default null,
    p_globalization_assignee    in varchar2  default null,
    --
    -- user interface
    --
    p_user_interface_impact     in varchar2  default null,
    p_user_interface_status     in number    default null,
    p_user_interface_assignee   in varchar2  default null
    );

procedure delete_feature (
    p_id in number );

procedure create_feature_map (
   p_id                                number,
   p_feature_id                        number default null,
   p_dependent_on_feature_id           number default null );

--==============================================================================
-- APIs for import
--==============================================================================

procedure create_news (
    p_id                in number,
    p_news_text         in varchar2 default null,
    p_created_by        in varchar2 default null,
    p_created_on        in date     default null,
    p_last_updated_by   in varchar2 default null,
    p_last_updated_on   in date     default null );

procedure create_link (
    p_id                in number,
    p_link_name         in varchar2 default null,
    p_feature_id        in number   default null,
    p_link_target       in varchar2 default null,
    p_link_category     in varchar2 default null,
    p_link_desc         in varchar2 default null,
    p_tags              in varchar2 default null,
    p_created_by        in varchar2 default null,
    p_created_on        in date     default null,
    p_updated_by        in varchar2 default null,
    p_updated_on        in date     default null );

procedure create_bug (
    p_id                        in number,
    p_bug_id                    in number   default null,
    p_product                   in varchar2 default null,
    p_product_version           in varchar2 default null,
    p_assigned_to               in varchar2 default null,
    p_component                 in varchar2 default null,
    p_bug_severity              in number   default null,
    p_bug_status                in number   default null,
    p_priority                  in number   default null,
    p_fix_by_release            in varchar2 default null,
    p_tags                      in varchar2 default null,
    p_estimated_fix_date        in date     default null,
    p_actual_fix_date           in date     default null,
    p_bug_title                 in varchar2 default null,
    p_bug_description           in clob     default null,
    p_reported_platform         in varchar2 default null,
    p_reported_browser          in varchar2 default null,
    p_reported_operating_system in varchar2 default null,
    p_application_id            in number   default null,
    p_page_id                   in number   default null,
    p_impact                    in varchar2 default null,
    p_feature_id                in number   default null,
    p_target_milestone_id       in number   default null,
    p_task_id                   in number   default null,
    p_duplicate_of_bug          in number   default null,
    p_customer_name             in varchar2 default null,
    p_customer_issue            in clob     default null,
    p_created_by                in varchar2 default null,
    p_created_on                in date     default null,
    p_updated_by                in varchar2 default null,
    p_updated_on                in date     default null );

procedure create_todo (
    p_id                        in  number,
    p_friendly_id               in  number   default null,
    p_assigned_to               in  varchar2 default null,
    p_contributor               in  varchar2 default null,
    p_task_name                 in  varchar2 default null,
    p_description               in  clob     default null,
    p_task_status               in  number   default null,
    p_task_category             in  varchar2 default null,
    p_task_tags                 in  varchar2 default null,
    p_start_date                in  date     default null,
    p_due_date                  in  date     default null,
    p_date_completed            in  date     default null,
    p_application_id            in  number   default null,
    p_page_id                   in  number   default null,
    p_websheet_id               in  number   default null,
    p_product_version           in  varchar2 default null,
    p_ref_component_type        in  varchar2 default null,
    p_customer_details          in  varchar2 default null,
    p_event_id                  in  number   default null,
    p_feature_id                in  number   default null,
    p_parent_task_id            in  number   default null,
    p_task_group                in  varchar2 default null,
    p_estimated_effort_in_hours in  number   default null,
    p_release                   in  varchar2 default null,
    p_security_group_id         in  number   default null,
    p_created_by                in  varchar2 default null,
    p_created_on                in  date     default null,
    p_updated_by                in  varchar2 default null,
    p_updated_on                in  date     default null );

procedure create_event (
    p_id                          in number,
    p_event_id                    in number   default null,
    p_event_name                  in varchar2 default null,
    p_event_date                  in date     default null,
    p_event_type                  in varchar2 default null,
    p_release                     in varchar2 default null,
    p_event_desc                  in clob     default null,
    p_tags                        in varchar2 default null,
    p_event_owner                 in varchar2 default null,
    p_selectable_from_features_yn in varchar2 default null,
    p_created_by                  in varchar2 default null,
    p_created_on                  in date     default null,
    p_updated_by                  in varchar2 default null,
    p_updated_on                  in date     default null );

procedure create_feature (
    p_id                        in number,
    p_feature_id                in number   default null,
    p_feature_type_id           in number   default null,
    p_feature_name              in varchar2 default null,
    p_feature_owner             in varchar2 default null,
    p_feature_contributor       in varchar2 default null,
    p_focus_area                in varchar2 default null,
    p_release                   in varchar2 default null,
    p_feature_desc              in clob     default null,
    p_justification             in varchar2 default null,
    p_feature_tags              in varchar2 default null,
    p_feature_priority          in number   default null,
    p_feature_status            in number   default null,
    p_feature_desirability      in number   default null,
    p_due_date                  in date     default null,
    p_start_date                in date     default null,
    p_module                    in varchar2 default null,
    p_estimated_effort_in_hours in number   default null,
    p_publishable_yn            in varchar2 default null,
    p_publishable_description   in varchar2 default null,
    p_globalization_impact      in varchar2 default null,
    p_globalization_assignee    in varchar2 default null,
    p_globalization_status      in number   default null,
    p_user_interface_impact     in varchar2 default null,
    p_user_interface_assignee   in varchar2 default null,
    p_user_interface_status     in number   default null,
    p_doc_impact                in varchar2 default null,
    p_doc_status                in number   default null,
    p_doc_writer                in varchar2 default null,
    p_testing_impact            in varchar2 default null,
    p_testing_assignee          in varchar2 default null,
    p_testing_status            in number   default null,
    p_security_impact           in varchar2 default null,
    p_security_assignee         in varchar2 default null,
    p_security_status           in number   default null,
    p_accessibility_impact      in varchar2 default null,
    p_accessibility_assignee    in varchar2 default null,
    p_accessibility_status      in number   default null,
    p_application_id            in number   default null,
    p_parent_feature_id         in number   default null,
    p_event_id                  in number   default null,
    p_created_by                in varchar2 default null,
    p_created_on                in date     default null,
    p_updated_by                in varchar2 default null,
    p_updated_on                in date     default null );

procedure create_feature_history (
    p_id                 in number,
    p_feature_id         in number   default null,
    p_column_name        in varchar2 default null,
    p_column_data_type   in varchar2 default null,
    p_old_value_varchar2 in varchar2 default null,
    p_new_value_varchar2 in varchar2 default null,
    p_old_value_clob     in clob     default null,
    p_new_value_club     in clob     default null,
    p_created_by         in varchar2 default null,
    p_created_on         in date     default null );

procedure create_feature_progress (
    p_id                in number,
    p_feature_id        in number   default null,
    p_progress_note     in varchar2 default null,
    p_created_by        in varchar2 default null,
    p_created_on        in date     default null,
    p_updated_by        in varchar2 default null,
    p_updated_on        in date     default null );

procedure create_task (
    p_id                 in number,
    p_friendly_id        in number   default null,
    p_assigned_to        in varchar2 default null,
    p_contributor        in varchar2 default null,
    p_task_name          in varchar2 default null,
    p_description        in clob     default null,
    p_task_status        in number   default null,
    p_task_category      in varchar2 default null,
    p_task_tags          in varchar2 default null,
    p_start_date         in date     default null,
    p_date_completed     in date     default null,
    p_application_id     in number   default null,
    p_websheet_id        in number   default null,
    p_product_version    in varchar2 default null,
    p_ref_component_type in varchar2 default null,
    p_customer_details   in varchar2 default null,
    p_event_id           in number   default null,
    p_feature_id         in number   default null,
    p_created_by         in varchar2 default null,
    p_created_on         in date     default null,
    p_updated_by         in varchar2 default null,
    p_updated_on         in date     default null,
    p_parent_task_id     in number   default null,
    p_task_group         in varchar2 default null,
    p_page_id            in number   default null,
    p_release            in varchar2 default null,
    p_due_date           in date     default null,
    p_estimated_effort_in_hours in number default null
 );

procedure create_task_progress (
    p_id                in number,
    p_task_id           in number   default null,
    p_progress_note     in varchar2 default null,
    p_created_by        in varchar2 default null,
    p_created_on        in date     default null,
    p_updated_by        in varchar2 default null,
    p_updated_on        in date     default null );

procedure create_task_defaults (
    p_id                          in number,
    p_default_release             in varchar2 default null,
    p_default_assignee            in varchar2 default null,
    p_default_priority            in number   default null,
    p_default_rpt_feature_type_id in number   default null,
    p_show_all_or_open            in varchar2 default null,
    p_enable_feature_ui_yn        in varchar2 default null,
    p_enable_feature_test_yn      in varchar2 default null,
    p_enable_feature_doc_yn       in varchar2 default null,
    p_enable_feature_glob_yn      in varchar2 default null,
    p_enable_feature_sec_yn       in varchar2 default null,
    p_enable_feature_acc_yn       in varchar2 default null,
    p_created_by                  in varchar2 default null,
    p_created_on                  in date     default null,
    p_updated_by                  in varchar2 default null,
    p_updated_on                  in date     default null );


procedure create_feedback (
    p_id                        in number,
    p_feedback_id               in number                   default null,
    p_feedback_comment          in varchar2                 default null,
    p_feedback_type             in number                   default null,
    p_feedback_status           in number                   default null,
    p_developer_comment         in varchar2                 default null,
    p_public_response           in varchar2                 default null,
    p_application_id            in number                   default null,
    p_application_name          in varchar2                 default null,
    p_page_id                   in number                   default null,
    p_page_name                 in varchar2                 default null,
    p_page_last_updated_by      in varchar2                 default null,
    p_page_last_updated_on      in date                     default null,
    p_session_id                in varchar2                 default null,
    p_apex_user                 in varchar2                 default null,
    p_user_email                in varchar2                 default null,
    p_application_version       in varchar2                 default null,
    p_session_info              in varchar2                 default null,
    p_session_state             in varchar2                 default null,
    p_parsing_schema            in varchar2                 default null,
    p_screen_width              in varchar2                 default null,
    p_screen_height             in varchar2                 default null,
    p_http_user_agent           in varchar2                 default null,
    p_remote_addr               in varchar2                 default null,
    p_remote_user               in varchar2                 default null,
    p_http_host                 in varchar2                 default null,
    p_server_name               in varchar2                 default null,
    p_server_port               in varchar2                 default null,
    p_tags                      in varchar2                 default null,
    p_label_01                  in varchar2                 default null,
    p_label_02                  in varchar2                 default null,
    p_label_03                  in varchar2                 default null,
    p_label_04                  in varchar2                 default null,
    p_label_05                  in varchar2                 default null,
    p_label_06                  in varchar2                 default null,
    p_label_07                  in varchar2                 default null,
    p_label_08                  in varchar2                 default null,
    p_attribute_01              in varchar2                 default null,
    p_attribute_02              in varchar2                 default null,
    p_attribute_03              in varchar2                 default null,
    p_attribute_04              in varchar2                 default null,
    p_attribute_05              in varchar2                 default null,
    p_attribute_06              in varchar2                 default null,
    p_attribute_07              in varchar2                 default null,
    p_attribute_08              in varchar2                 default null,
    p_bug_id                    in number                   default null,
    p_feature_id                in number                   default null,
    p_task_id                   in number                   default null,
    p_logging_email             in varchar2                 default null,
    p_logging_security_group_id in varchar2                 default null,
    p_logged_by_workspace_name  in varchar2                 default null,
    p_deployment_system         in varchar2                 default null,
    p_created_by                in varchar2                 default null,
    p_created_on                in timestamp with time zone default null,
    p_updated_by                in varchar2                 default null,
    p_updated_on                in timestamp with time zone default null );

procedure create_feedback_followup (
    p_id          in number,
    p_feedback_id in number                   default null,
    p_follow_up   in varchar2                 default null,
    p_email       in varchar2                 default null,
    p_created_by  in varchar2                 default null,
    p_created_on  in timestamp with time zone default null,
    p_updated_by  in varchar2                 default null,
    p_updated_on  in timestamp with time zone default null );

/* Not yet used
procedure create_team_file (
    p_id                 in number,
    p_file_type          in varchar2 default null,
    p_feature_id         in number   default null,
    p_task_id            in number   default null,
    p_event_id           in number   default null,
    p_feedback_id        in number   default null,
    p_bug_id             in number   default null,
    p_file_name          in varchar2 default null,
    p_file_mime_type     in varchar2 default null,
    p_file_character_set in varchar2 default null,
    p_file_last_update   in date     default null,
    p_created_on         in date     default null,
    p_created_by         in date     default null );
*/

procedure create_team_tags (
    p_id                               in number      default null,
    p_tag                              in varchar2    default null,
    p_tag_type                         in varchar2    default null,
    p_component_id                     in number      default null );

end wwv_flow_team_api;
/
show errors
