/*!
 *
 * JET Chart widget
 * Copyright (c) 1999, 2017, Oracle and/or its affiliates. All rights reserved.
 *
 */
/*global requirejs,$v,require,apex */
( function( $, server, util, debug, region ) {
    "use strict";

    apex.widget.jet = {
        init: function( pRegionId, pWidth, pHeight, pOptions, pSpark, pRefreshInterval, pApexAjaxIdentifier ) {
            require([ "ojs/ojcore", "jquery", "ojs/ojchart", "ojs/ojgauge", "ojs/ojvalidation" ], function( oj ) {
                    var lRegion$ = $( "#" + util.escapeCSS( pRegionId ) + "_jet" , apex.gPageContext$ ),
                    lOptions = pOptions || {};

                /*
                 * Formatting Converter Handling - used to create a Converter Factory object, required for applying datetime and numeric formatting to the JET chart data
                 *
                 * var lType:  -) number: Converter type to be used for decimal/percent/currency formatting
                 *             -) datetime: Converter type to be used for date, time, datetime formatting
                 */
                // Value Format
                if ( lOptions.valueFormats ) {
                    $.each( lOptions.valueFormats, function ( i ) {
                        $.each( lOptions.valueFormats[ i ], function ( j, v ) {
                            if ( j === "converter" ) {
                                var lType = v.formatType ? "datetime" : "number";
                                lOptions.valueFormats[ i ][ j ] = oj.Validation.converterFactory( lType ).createConverter( lOptions.valueFormats[ i ][ j ] );
                            }
                        });
                    });
                }

                //X Axis
                if ( lOptions.xAxis ) {
                    if ( lOptions.xAxis.tickLabel ) {
                        $.each( lOptions.xAxis.tickLabel, function ( i, v ) {
                            if ( i === "converter" ) {
                                var lType = v.formatType ? "datetime" : "number";
                                lOptions.xAxis.tickLabel[ i ] = oj.Validation.converterFactory( lType ).createConverter( lOptions.xAxis.tickLabel[ i ] );
                            }
                        });
                    }
                }

                // Y Axis
                if ( lOptions.yAxis ) {
                    if ( lOptions.yAxis.tickLabel ) {
                        $.each( lOptions.yAxis.tickLabel, function ( i, v ) {
                            if ( i === "converter" ) {
                                var lType = v.formatType ? "datetime" : "number";
                                lOptions.yAxis.tickLabel[ i ] = oj.Validation.converterFactory( lType ).createConverter( lOptions.yAxis.tickLabel[ i ] );
                            }
                        });
                    }
                }

                // Y2 Axis
                if ( lOptions.y2Axis ) {
                    if ( lOptions.y2Axis.tickLabel ) {
                        $.each( lOptions.y2Axis.tickLabel, function ( i, v ) {
                            if ( i === "converter" ) {
                                var lType = v.formatType ? "datetime" : "number";
                                lOptions.y2Axis.tickLabel[i] = oj.Validation.converterFactory( lType ).createConverter( lOptions.y2Axis.tickLabel[i] );
                            }
                        });
                    }
                }

                // Initialize Chart
                if ( lOptions.type === "dial" )	{
                    lRegion$.ojDialGauge( lOptions );
                } else if ( pSpark ) {
                    lRegion$.ojSparkChart( lOptions );
                } else {
                    lRegion$.ojChart( lOptions );
                }

                // Chart Resizing
                // todo consider that this resize method is not needed or only needs to be called once.
                function resize() {
                    var lWidth   = "100%",
                        lHeight  = null;

                    if ( pWidth ) {
                        lWidth = pWidth + "px";
                    }
                    if ( pHeight ) {
                        lHeight = pHeight + "px";
                    }
                    lRegion$.css( "width", "100%" );
                    lRegion$.css( "max-width", lWidth );
                    if ( lHeight ) {
                        lRegion$.css( "height", lHeight );
                    }
                    if ( !$.mobile ) {
                        lRegion$.css( "min-width", "100px" );
                    }
                } //resize

                // Register Chart with our region interface
                region.create( pRegionId, {
                    type: "JetChart",
                    refresh: function() {
                        resize();
                        refresh();
                    },
                    focus: function() {
                        lRegion$.focus();
                    },
                    widget: function() {
                        return lRegion$;
                    }
                });
                region( pRegionId ).refresh();

                // Uses AJAX to get the newest chart data
                function refresh() {
                    server.plugin( pApexAjaxIdentifier, {
                        pageItems: lOptions.pageItems
                    }, {
                        dataType:                 "json",
                        refreshObject:            lRegion$,
                        loadingIndicator:         lRegion$,
                        loadingIndicatorPosition: "centered",
                        success:                  showResult
                    });
                } // refresh

                // AJAX success callback to set the chart data
                function showResult( pData ) {
                    var C_EXCLUDE_CHARTS = [ "pie", "donut", "funnel", "pyramid", "scatter", "bubble" ],
                        i, ans,
                        lChartType  = ( lOptions.type === "dial" ) ? "dial" : "chart",
                        type        = pSpark ? "spark" : lChartType;

                    // helper function: retrieve links from series data, to associate a link action with a specific data point in the JET chart data
                    function getLinks( obj, key, val ) {
                        var i;

                        function linkFound( obj, k, v ) {
                            var i;
                            for( i = 0; i < obj.length; i++ ) {
                                if ( lOptions.type === "pie" || lOptions.type === "donut" || lOptions.type === "funnel" ) {
                                    // Multi-series query for Pie, Donut, and Funnel should match by key
                                    if ( obj[ i ].name === k ) {
                                        return obj[ i ].link;
                                    }
                                } else if ( lOptions.type === "stock" ) {
                                    var d = new Date( obj[ i ].name );
                                    var epochTime = d.getTime();
                                    // Multi-series query for Pie & Donut should match by val
                                    if ( epochTime === v || d === v ) {
                                        return obj[ i ].link;
                                     }
                                }  else {
                                    if ( obj[ i ].name === v ) {
                                        return obj[ i ].link;
                                    } else {
                                        var dt = new Date( obj[ i ].name );
                                        var epochTimeVal = dt.getTime();
                                        // Multi-series query for Pie & Donut should match by val
                                        if ( epochTimeVal === v || dt === v ) {
                                            return obj[ i ].link;
                                        }
                                    }
                                }
                            }
                        } // linkFound

                        for ( i in obj ) {
                            if ( obj.hasOwnProperty( i ) ) {
                                if ( obj[ i ].id ) {  // Static ID-identified series mapping
                                    if (( obj[ i ].id === key || obj[ i ].name === key ) && obj[ i ].items.length > 0 ) {
                                        return linkFound( obj[ i ].items, key, val );
                                    }
                                } else if ( obj[ i ].name ) { // Series name mapping
                                    if ( obj[ i ].name === key && obj[ i ].items.length > 0 ) {
                                        return linkFound( obj[ i ].items, key, val );
                                    }
                                }
                            }
                        }
                    }  // getLinks

                    // Check for developer-provided callback, via the chart-level JavaScript Code attribute, to manipulate the fetched data
                    if ( $.isFunction( lOptions.dataFilter )) {
                        pData = lOptions.dataFilter( pData );
                    }

                    // helper function: Sort the data, to ensure the order of the series items matches the groups array
                    function sortArray( values ) {
                        values.sort( function( a, b ) {
                            if ( a.name < b.name ) {
                                return -1;
                            } else if ( a.name > b.name ) {
                                return 1;
                            }
                            return 0;
                        });
                    } // sortArray

                    // helper function: Fill gaps for missing data points, to ensure each group has an associated data point in each series
                    function fillGaps( groups, items ) {

                        sortArray( items );

                        for ( var groupIdx = 0; groupIdx < groups.length; groupIdx++ ) {

                            // Each group entry must have a corresponding entry in the items array, required by JET
                            if ( !items[ groupIdx ] || items[ groupIdx ].name !== groups[ groupIdx ].name ) {
                                // Add an entry for a missing data point
                                items.splice( groupIdx, 0, { name: groups[ groupIdx ].name , value: 0 } );
                            } else if ( items[ groupIdx ].id !== groupIdx ) {
                                // Correct the id if we have added new data points
                                items[ groupIdx ].id = groupIdx;
                            }
                        }
                    } // fillGaps

                    // Dial Chart Title substitution of value/max data references and numeric converter handling for metricLabel data
                    if ( lChartType === "dial" ) {
                        if ( lOptions.title ) {
                            lRegion$.closest( "div" ).attr( "title", lOptions.title.replace( /#VALUE#/g, pData.value ).replace( /#MAX#/g, pData.max ));
                        }
                        if ( pData.metricLabel ) {
                            $.each( pData.metricLabel, function ( i ) {
                                if ( i === "converter" ) {
                                    pData.metricLabel[ i ] = oj.Validation.converterFactory( "number" ).createConverter( pData.metricLabel[ i ] );
                                }
                            });
                        }
                    } else {
                        // Densify the chart data for a multi-series chart.
                        // Exceptions 1) chart types: Dial Gauge, Pie, Donut, Funnel, Pyramid, Scatter, Bubble
                        //            2) charts using the Time Axis Type settings of 'Mixed Frequency' or 'Skip Gaps' - for backward compatibility with 5.1.0 - 5.1.2.
                        if ( $.inArray( lOptions.type, C_EXCLUDE_CHARTS ) === -1 ) {
                            if ( pData.series.length > 1 && ( lOptions.timeAxisType !== "mixedFrequency" && lOptions.timeAxisType !== "skipGaps" ) ) {
                                //if ( pData.series.length > 1 ) {

                                sortArray( pData.groups );

                                for ( var seriesIdx = 0; seriesIdx < pData.series.length; seriesIdx++ ) {
                                    fillGaps( pData.groups, pData.series[ seriesIdx ].items );
                                }
                            }
                        }
                    }

                    type = { spark: "ojSparkChart", chart: "ojChart", dial: "ojDialGauge"}[ type ];
                    // Add Chart Data
                    lRegion$[ type ]( pData );

                    // Handle Visibility of Chart, when changed from hidden to visible
                    apex.widget.util.onVisibilityChange( lRegion$[ 0 ], function( show )  {
                        if ( show ) {
                            // Inform JET of visibility of previously hidden component
                            oj.Components.subtreeShown( lRegion$[ 0 ] );
                            // Resize collapsed region where no height defined on region or via template options
                            resize();
                            lRegion$[ type ]( "refresh" );
                        } else {
                            // Inform JET of visibility of newly hidden component
                            oj.Components.subtreeHidden( lRegion$[ 0 ] );
                        }
                    } );

                    // Process Links on Data Points - Only Supported by ojChart charts
                    if ( type === "ojChart" ){
                        lRegion$[ type ]( {
                            "drill": function( event, ui ) {
                                if ( ( ui.group ) && ( ui.series ) ) {
                                    ans = getLinks( pData.series, ui.series, ui.group );
                                    if ( ans ) {
                                        debug.log( "Chart link: " + ans );
                                        apex.navigation.redirect( ans );
                                    }
                                }
                            }
                        });
                    }

                    // Automatic Refresh Handling
                    if ( pRefreshInterval > 0 ) {
                        setTimeout(function () {
                            refresh();
                        }, pRefreshInterval * 1000);
                    }
                } // showResult

                apex.widget.jet.getChartJSON = function () {
                    return lOptions;
                }; // getChartJSON

            });
        }
    };
})( apex.jQuery, apex.server, apex.util, apex.debug, apex.region );